#!/usr/bin/perl -w -I/usr/sausalito/perl/

use POSIX;
use CCE;
use strict;

$CCE::DEBUG = 2;

# Globals.
my $Openssl_cmd = "/usr/sbin/openssl";
my $CertValidFor = 9999; # Number of days self signed certs are valid for.
my $ResponsibleUser = "admin"; # User responsible for ssl managment.
my $CertDir = "/etc/httpd/ssl/";

my $cce = new CCE ( Domain => 'base-ssl' );
my $errors;

$cce->connectfd(\*STDIN,\*STDOUT);

# No need to validate as all identity functions are already validated by
# identity.pl and all hostname functions are already validated by system.


if( ! validate($cce) ) {
	$cce->bye("FAIL");
} else {
	set_identity($cce);
	$cce->bye("SUCCESS");
}

sub validate {
	my $cce = shift;
	my $errors;

	if( ! -d $CertDir ) {
		$cce->warn("certDirMissing");
		$errors ++;
	}

	if( ! -x $Openssl_cmd ) {
		$cce->warn("couldntRunOpenssl");
		$errors++;
	}

	return ( ! $errors );
}
	

sub set_identity
{
	my $cce = shift;

	my ($ret,$SysObj) = $cce->get($cce->event_oid());
	my ($reti,$IdentObj) = $cce->get($cce->event_oid(),"Identity");

	# We're starting from scratch here.. Make backups if we're going
	# to be overwriting something.
	for my $file ( qw(request certificate key) ) {
		if( -f "$CertDir/$file") {
			rename("$CertDir/$file","$CertDir/$file.bak");
		}
	}

	# Fork of here as generating certificates takes too long.
	my $pid = fork();
	if( $pid ) {
		return 1;
	}

	# daemonize myself
	close(STDIN); close(STDOUT); close(STDERR);
	my $logfile = "/tmp/ssl.log.$$";
	open(STDOUT, ">$logfile");
	open(STDERR, ">&STDOUT");
	open(STDIN, "</dev/null");
	POSIX::setsid();

	# generate key:
	system("$Openssl_cmd genrsa -out $CertDir/key 1024 1>&2");

	# generate certificate request:
	my $cmnd =  "|$Openssl_cmd req -new -config /usr/lib/openssl.cnf "
		. "-key $CertDir/key -days $CertValidFor -out $CertDir/request 1>&2";

	open(REQ, $cmnd);

	print REQ $IdentObj->{country} ."\n";
	print REQ $IdentObj->{state} . "\n";
	print REQ $IdentObj->{locality} . "\n";
	print REQ $IdentObj->{organisation} . "\n";
	print REQ $IdentObj->{organisationUnit} . "\n";
	print REQ $SysObj->{hostname} .'.'. $SysObj->{domainname} . "\n";
	print REQ "$ResponsibleUser@".$SysObj->{hostname}.'.'. $SysObj->{domainname}. "\n";
	close REQ;	

	# self-sign certificate request to make certificate
	system("$Openssl_cmd x509 -days $CertValidFor -req -signkey $CertDir/key "
		  ."-in $CertDir/request -out $CertDir/certificate 1>&2");

	chmod 0660, "$CertDir/key";
	chmod 0660, "$CertDir/certificate";
	chmod 0660, "$CertDir/request";

	exit 1;
}

1;
